package trz.gaia;

import java.util.*;

/**
L' insieme di tutte le cellule che partecipano a Gaia. Con partecipare si intende semplicemente
essere nella stessa rete (a livello di link layer).
La partecipazione attiva (apertura connessioni-gaia, routing, ecc.) viene decisa da ogni singola cellula
con i metodi entra e esci.

@see trz.gaia.Cellula
*/
public abstract class Gaia {

	Vector tickListener = new Vector();
	Vector gaiaListener = new Vector();
	//qui conservo solo la lista, la gestione vera e propria e' fatta dalle connessioni
	Vector pcktListener = new Vector();
	
	int tickCount = 0;
	
	/**
	Fa avanzare la simulazione di un "tick" 
	*/
	public void tick() {

		//esegue tutto quello che deve
		main();
		incTickCount();
		notifyTickListener(new TickEvent(getTickCount()));
	}

	//il corpo principale di esecuzione
	protected abstract void main();
	
	//sospetto che il synchronized non serva
	private synchronized void incTickCount() {
		tickCount++;
	}
	
	//sospetto che il synchronized non serva
	protected synchronized void resetTickCount() {
		tickCount = 0;
	}
	
	/**
	Ritorna il conteggio dei tick
	*/
	public synchronized int getTickCount() {
		return tickCount;
	}
	
	/**
	Aggiunge una cellula.
	*/
	public abstract void addCellula(Cellula cellula);

	/**
	Aggiunge una cellula ad un particolare gruppo. Cosa sia e come funzioni un gruppo
	dipende dalla particolare implementazione. 
	*/
	public abstract void addCellula(Cellula cellula, String nomeGruppo);

	/**
	Rimuove la cellula passata
	*/	
	public abstract void removeCellula(Cellula cellula);

	/**
	Resetta la simulazione
	*/
	public abstract void reset();

	//GESTIONE ASCOLTATORI

	//tick event
	public void addTickListener(TickListener listener) {
		tickListener.add(listener);
	}
	public void removeTickListener(TickListener listener) {
		tickListener.remove(listener);
	}
	private void notifyTickListener(TickEvent e) {
		for (int i = 0; i < tickListener.size(); i++)
			((TickListener)tickListener.elementAt(i)).tickEvent(e);
	}

	//gaia event
	public void addGaiaListener(GaiaListener listener) {
		gaiaListener.add(listener);
	}
	public void removeGaiaListener(GaiaListener listener) {
		gaiaListener.remove(listener);
	}
	private void notifyGaiaListener(GaiaEvent e) {
		for (int i = 0; i < gaiaListener.size(); i++)
			((GaiaListener)gaiaListener.elementAt(i)).gaiaEvent(e);
	}
	
} 
