package trz.gaia.rna;

import trz.gaia.*;
import trz.gaia.util.*;
import java.util.*;
import java.io.*;

/**
Rappresenta una connessione dal punto di vista di una sola cellula.
(cioe' la connessione non e' comune tra le due cellule: ognuna ha la propria).
Una connessione non cambia mai le sue estremita'
Permette di spedire e ricevere pacchetti.
*/

public class ConnessioneLocale extends AbstractConnessione {

	Vector coda = new Vector();
	
	Cellula cellulaFrom;	//questo e' un riferimento alla cellula che
							//contiene la con
	String nomeCellulaTo;	//l' altra estremita'
	LinkLayer link = null;
	
	/**
	Apre un "canale" verso un' altra cellula. L' altra cellula puo' anche non esistere
	In pratica e' come spedire un SYN. Questa non e' davvero una delle N connessioni. Lo
	puo' diventare dopo che si e' fatto un qualche tipo di handshake.
	*/
	//### Vedere se si trova un modo piu' bello per gestire il tutto.
	//forse e' meglio fare 2 classi, una di handshake che se va a buon fine restituisce una
	//connessione gia' pronta.
	public ConnessioneLocale(Cellula cellulaFrom, String nomeCellulaTo) throws IOException {

		this.cellulaFrom = cellulaFrom;
		this.nomeCellulaTo = nomeCellulaTo;
		link = LinkLayer.getInstance();
	}
	
	/**
	Il nome dell' estremita' di partenza
	*/
	public String getNomeFrom() {
		return cellulaFrom.getNome();
	}

	/**
	Chiude la connessione
	*/
	public void close() {
		
		link.closeConnessione(this);
	}

	/**
	Il nome dell' estremita' di arrivo
	*/
	public String getNomeTo() {
		return nomeCellulaTo;
	}
	
	/**
	Costruisce un pacchetto con quei dati e lo spedisce su questa connessione
	*/
	/*
	I pacchetti vengono analizzati e inseriti nella coda del destinatario (gestendo
	la simulazione di pacchetti persi e simili).
	*/
	public void write(String dati) throws IOException {

		Packet pckt = new Packet(cellulaFrom.getNome(), nomeCellulaTo);
		pckt.setDati(dati);
		write(pckt);
	}
	
	//spedisce il pacchetto
	public void write(Packet pckt) throws IOException {

		PacketEvent pe = new PacketEvent(pckt, PacketEvent.SPEDITO);
		notifyPacketSent(pe);
		link.send(pckt);
	}

	/**
	Legge un pacchetto dalla connessione. Ritorna null se non ci sono pacchetti
	*/	
	public Packet read() {

		Packet pckt = null;
		if (coda.size() > 0) {
			pckt = (Packet)coda.remove(0);

			PacketEvent pe = new PacketEvent(pckt, PacketEvent.RICEVUTO);
			notifyPacketReceived(pe);
		}
		return pckt;
	}

	/**
	Aggiunge un pacchetto nella coda di questa connessione
	*/
	//questo e' chiamato dall' altro thread (attraverso il link layer)
	public void putPacket(Packet pckt) {
		//carico nella coda
		//### forse e' meglio mettere qui l' evento packet received ???
		coda.add(pckt);
	}
	
	public String toString() {
		return cellulaFrom.getNome() + " -> " + nomeCellulaTo;
	}
}
