package trz.gaia.rna;

import trz.gaia.*;
import trz.gaia.util.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
Rappresenta una connessione dal punto di vista di una sola cellula.
(cioe' la connessione non e' comune tra le due cellule: ognuna ha la propria).
Una connessione non cambia mai le sue estremita'
Permette di spedire e ricevere pacchetti.
*/

public class ConnessioneRemota extends AbstractConnessione {

	Vector coda = new Vector();
	
	Cellula cellulaFrom;	//questo e' un riferimento alla cellula che
							//contiene la con
	String nomeCellulaTo;	//l' altra estremita'
	LinkLayer link = null;

	Socket s = null;
	//ObjectInputStream in = null;
	//ObjectOutputStream out = null;
	InputStream in = null;
	OutputStream out = null;
	
	/**
	Apre un "canale" verso un' altra cellula. L' altra cellula puo' anche non esistere
	In pratica e' come spedire un SYN. Questa non e' davvero una delle N connessioni. Lo
	puo' diventare dopo che si e' fatto un qualche tipo di handshake.
	*/
	//### Vedere se si trova un modo piu' bello per gestire il tutto.
	//forse e' meglio fare 2 classi, una di handshake che se va a buon fine restituisce una
	//connessione gia' pronta.
	public ConnessioneRemota(Cellula cellulaFrom, String nomeCellulaTo) throws IOException {

		this.cellulaFrom = cellulaFrom;
		this.nomeCellulaTo = nomeCellulaTo;
		link = LinkLayer.getInstance();
		//System.out.println("prima");
		//lo ricevo con dabati un _
		s = new Socket(nomeCellulaTo.substring(1), 11277);
		
		//qui devo leggere il primo carattere per capire se ha accettato o meno
		//si', e' davvero una schifezza, ma e' solo per test
		in = s.getInputStream();
		int res = in.read();
		if (res != 0) {
			in = null;
			throw new ConnectException("Connessione rifiutata: " + res);
		}
		//TEMP TEMP
		s.setSoTimeout(250);
		out = s.getOutputStream();
		System.out.println("esco");
	}
	
	/**
	Il nome dell' estremita' di partenza
	*/
	public String getNomeFrom() {
		return cellulaFrom.getNome();
	}

	/**
	Chiude la connessione
	*/
	public void close() {
		
		//link.closeConnessione(this);
		try {
			s.close();
		}catch(IOException e){}
	}

	/**
	Il nome dell' estremita' di arrivo
	*/
	public String getNomeTo() {
		return nomeCellulaTo;
	}
	
	/**
	Costruisce un pacchetto con quei dati e lo spedisce su questa connessione
	*/
	/*
	I pacchetti vengono analizzati e inseriti nella coda del destinatario (gestendo
	la simulazione di pacchetti persi e simili).
	*/
	public void write(String dati) throws IOException {

		Packet pckt = new Packet(cellulaFrom.getNome(), nomeCellulaTo);
		pckt.setDati(dati);
		write(pckt);
	}
	
	//spedisce il pacchetto
	public void write(Packet pckt) throws IOException {

		//System.out.println("scrivo");
		out.write((pckt.getDati()+ "-").getBytes());
		//System.out.println("scritto");
		//PacketEvent pe = new PacketEvent(pckt, PacketEvent.SPEDITO);
		//notifyPacketSent(pe);
		//link.send(pckt);
	}

	/**
	Legge un pacchetto dalla connessione. Ritorna null se non ci sono pacchetti
	Sistemare le exception.
	*/
	StringBuffer sb = new StringBuffer();
	public Packet read() {

		//System.out.println("leggo");
		Packet pckt = null;
		try {
			//System.out.println("leggo");
			int c = in.read();
			//System.out.println("trovo " + c);
			if (c != -1) {
				
				//System.out.println("trovo " + (char)c);
				if (((char)c) == '-') {

					//System.out.println("letto pckt " + sb.toString());
					pckt = new Packet(nomeCellulaTo, cellulaFrom.getNome());
					pckt.setDati(sb.toString());
					sb.setLength(0);
					PacketEvent pe = new PacketEvent(pckt, PacketEvent.RICEVUTO);
					notifyPacketReceived(pe);
				}
				else
					sb.append((char)c);
			}
			//else
				//System.out.println("nulla");
		}catch(Exception e){
			pckt = null;	//non si sa mai
		}
		return pckt;
	}

	/**
	Aggiunge un pacchetto nella coda di questa connessione
	*/
	//questo e' chiamato dall' altro thread (attraverso il link layer)
	public void putPacket(Packet pckt) {
		//carico nella coda
		//### forse e' meglio mettere qui l' evento packet received ???
		//coda.add(pckt);
	}
	
	public String toString() {
		return cellulaFrom.getNome() + " -> " + nomeCellulaTo;
	}
}
