package trz.gaia.rna;

import trz.gaia.*;
import trz.gaia.util.*;
import java.util.*;
import java.io.*;
 
/**
Una classe che gestisce l' effettiva spedizione dei pacchetti da una cellula
all' altra.<br>
L' eventuale simulazione della perdita di pacchetti viene gestita a livello di
connessione.<br>
*/
//Impostato come un singleton.
//TEMP: public solo per i javadoc
public class LinkLayer {

	
	private static LinkLayer istanza;

	private LinkLayer(){}
	
	public static LinkLayer getInstance(){
		if (istanza == null)
			istanza = new LinkLayer();
		return istanza;
	}

	Hashtable cellule = new Hashtable();
	
	/**
	Connette la cellula al link layer
	*/
	public void connetti(Cellula cellula) {
		cellule.put(cellula.getNome(), cellula);
	}
	
	/**
	Disconnette la cellula dal link layer
	*/
	public void disconnetti(Cellula cellula) {
		cellule.remove(cellula.getNome());
	}
	
	/*
	### Qui e' da capire bene come fare.
	Visto che il link layer ha tutte le cellule, ogni volta che deve consegnare un packet a qualcuno
	potrebbe cercare il tizio, farsi dare le sue con e cercarne una col tizio che ha spedito il msg
	Almeno le con sono conservate solo nelle cellule.
	Pero' a questo punto avrei dei packet che vengono scartati a livello di link layer senza che nessuno
	se ne accorga. Pero' far salire alla cellula dei packet che arrivano da cellule con cui lei non ha
	nessuna con aperta non ha proprio senso.
	
	Al tempo stesso tutte le connessioni sono istanziate dal LinkLayer e quindi lui puo' tenersi
	un riferimento e andarle subito a recuperare.
	*/
	
	//ogni con all' interno e' individuata dalla concatenazione di nomeFrom - nomeTo
	Hashtable connessioni = new Hashtable();
	/**
	Gestisce l' handshake sul link layer e restituisce una connessione se va tutto bene,
	null altrimenti.
	La cellula a cui ci si connette (nomeTo) si trovera' anche lei con una con aperta
	*/
	//### trovare un modo migliore: e' facile sbagliare o barare
	//##far salire le exception ?
	public Connessione getConnessione(Cellula cellulaFrom, String nomeTo) {
		
		/*
		### qui, basandomi sulla ListaConnessioni, dovrei andare a vedere se nomeTo
		corrisponde ad una cellula locale o ad una remota e gestire diversamente
		l' apertura della connessione
		*/
		//TEMPORANEAMENTE vedo se il nome inizia con un _ e lo considero come il nome
		//dell' host
	if (nomeTo.startsWith("_")) {
		try {
			//Socket s = new Socket(nomeTo.substring(1), 11277);
			System.out.println(cellulaFrom.getNome() + ": apro socket con " + nomeTo);
			ConnessioneRemota con = new ConnessioneRemota(cellulaFrom, nomeTo);
			System.out.println(cellulaFrom.getNome() + ": aperto socket con " + nomeTo);
			return con;
		}catch(IOException e) {
			System.out.println(cellulaFrom.getNome() + ": con rifiutata da " + nomeTo + ": " + e);
		}
	}
	else {
		//creo una connessione locale
		Cellula cellulaTo = null;
		try {
			cellulaTo = (Cellula)cellule.get(nomeTo);
			Connessione conTo = new ConnessioneLocale(cellulaTo, cellulaFrom.getNome());
			
			int res = cellulaTo.accetta(conTo);
			if (res >= 0) {

				//registro la con che ho proposto all' altro (e che di la' e' stata salvata)
				//###e' davvero un pasticcio, ma al momento non importa
				connessioni.put(nomeTo + "-" + cellulaFrom.getNome(), conTo);

				//registro l' altra estremita' della connessione
				Connessione con = new ConnessioneLocale(cellulaFrom, nomeTo);
				connessioni.put(cellulaFrom.getNome() + "-" + nomeTo, con);
				//semaforoLink.free();
				return con;
			}
			//else
				//System.out.println(cellulaFrom.getNome() + ": con rifiutata da " + cellulaTo + ": " + res);
		}
		catch(IOException e){
			System.out.println(cellulaFrom.getNome() + ": con rifiutata da " + cellulaTo + ": " + e);
		}
	}//fine else
		return null;
	}

	/**
	Chiude la connessione
	*/
	public void closeConnessione(Connessione con) {
		connessioni.remove(con.getNomeFrom() + "-" + con.getNomeTo());
	}	
	/**
	Scrive un pckt a livello di network layer
	*/
	public void send(Packet pckt) throws IOException {
		
		Connessione con = (Connessione)connessioni.get(pckt.getNomeTo()+"-"+pckt.getNomeFrom());
		/*
		Qui c'e' il problema dei packet scartati senza che nessuno lo veda.
		Volendo si puo' mettere un LinkListener o simile.
		*/
		if (con == null)
			//###generare un exception, spedire un ICMP o cosa ?
			throw new IOException("Connessione con " + pckt.getNomeTo() + " chiusa");
		//TEMP TEMP
		//###sara' a rischio ???
		con.putPacket(pckt);
	}

	/*
	Non c'e' un metodo read perche' quando spedisco un pacchetto questo viene caricato
	direttamente nel buffer del destinatario.
	L' alternativa era che ogni cellula chiedesse al link layer i suoi pacchetti, ma questo 
	avrebbe voluto dire che, piu' o meno volontariamente, una cellula avrebbe potuto "sniffare"
	pacchetti altrui, e la cosa nin mi piaceva molto.
	*/

}
